Package: TwoStepSDFM
Title: Estimate a Sparse Mixed Frequency Gaussian Factor Model Using a
        Two-Step Procedure
Version: 0.2.2
Authors@R: person(
    given = "Domenic",
    family = "Franjic",
    email = "franjic@uni-hohenheim.de",
    role = c("aut", "cre")
	)
Maintainer: Domenic Franjic <franjic@uni-hohenheim.de>
Description: Estimate a sparse Gaussian state-space model with mixed frequency
    data via sparse principal components analysis and the Kalman filter
    and smoother. For more details see Franjic and Schweikert (2024) <doi:10.2139/ssrn.4733872>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: Rcpp (>= 1.0.8), zoo, xts, lubridate, ggplot2, patchwork,
        doSNOW, doParallel, foreach, parallel, Rdpack, grDevices, withr
RdMacros: Rdpack
LinkingTo: Rcpp, RcppEigen
NeedsCompilation: yes
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 4.0)
LazyData: TRUE
Packaged: 2026-05-19 08:26:16 UTC; 520K-User
Author: Domenic Franjic [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-19 08:50:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 11:18:22 UTC; unix
Archs: TwoStepSDFM.so.dSYM
