# Generated by roxygen2: do not edit by hand

S3method(plot,KFSFit)
S3method(plot,NoOfFactorsFit)
S3method(plot,SDFMFit)
S3method(plot,SDFMcrossVal)
S3method(plot,SDFMnowcast)
S3method(plot,SPCAFit)
S3method(plot,SimulData)
S3method(predict,SDFMFit)
S3method(print,KFSFit)
S3method(print,NoOfFactorsFit)
S3method(print,SDFMFit)
S3method(print,SDFMcrossVal)
S3method(print,SDFMnowcast)
S3method(print,SPCAFit)
S3method(print,SimulData)
export(crossVal)
export(kalmanFilterSmoother)
export(noOfFactors)
export(nowcast)
export(simFM)
export(sparsePCA)
export(twoStepDenseDFM)
export(twoStepSDFM)
import(doParallel)
import(doSNOW)
import(foreach)
import(ggplot2)
import(grDevices)
import(lubridate)
import(parallel)
import(patchwork)
import(stats)
import(utils)
import(withr)
import(xts)
import(zoo)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
useDynLib(TwoStepSDFM, .registration=TRUE)
