# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

runARDL <- function(target_variable, target_variable_predictor, predictor_variable, max_target_lags, max_predictor_lags, crit, jitter) {
    .Call(`_TwoStepSDFM_runARDL`, target_variable, target_variable_predictor, predictor_variable, max_target_lags, max_predictor_lags, crit, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. 
#'
runDL <- function(target_variable, predictor_variable, max_predictor_lags, crit, jitter) {
    .Call(`_TwoStepSDFM_runDL`, target_variable, predictor_variable, max_predictor_lags, crit, jitter)
}

runNoOfFactorsTest <- function(data_matrix_in, test_values, min_no_factors, max_no_factors, confidence_threshold) {
    .Call(`_TwoStepSDFM_runNoOfFactorsTest`, data_matrix_in, test_values, min_no_factors, max_no_factors, confidence_threshold)
}

runNoOfFactorsInfoCrit <- function(data_matrix_in, max_no_factors) {
    .Call(`_TwoStepSDFM_runNoOfFactorsInfoCrit`, data_matrix_in, max_no_factors)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runSPCA <- function(X_in, delay, selected, R, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, svd_method, normalise, comp_var_expl) {
    .Call(`_TwoStepSDFM_runSPCA`, X_in, delay, selected, R, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, svd_method, normalise, comp_var_expl)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runUVKFS <- function(X_in, delay, state_var_cov, measurement_var_cov, loading_matrix, factor_var_coefficient_matrices, R, order, fcast_horizon, decorr_errors, comp_null, parallel, jitter) {
    .Call(`_TwoStepSDFM_runUVKFS`, X_in, delay, state_var_cov, measurement_var_cov, loading_matrix, factor_var_coefficient_matrices, R, order, fcast_horizon, decorr_errors, comp_null, parallel, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runSDFMKFS <- function(X_in, delay, selected, R, order, decorr_errors, crit, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, fcast_horizon, jitter, svd_method) {
    .Call(`_TwoStepSDFM_runSDFMKFS`, X_in, delay, selected, R, order, decorr_errors, crit, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, fcast_horizon, jitter, svd_method)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepDFM()` instead for a stable and
#' supported interface.
#'
runDFMKFS <- function(X_in, delay, R, order, decorr_errors, crit, comp_null, parallel, fcast_horizon, jitter) {
    .Call(`_TwoStepSDFM_runDFMKFS`, X_in, delay, R, order, decorr_errors, crit, comp_null, parallel, fcast_horizon, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `SimFM()` instead for a stable and
#' supported interface.
#'
runStaticFM <- function(T, N, S, Lambda, mu_e, Sigma_e, A, order, quarterfy, corr, beta_param, m, seed, R, burn_in, rescale, parallel) {
    .Call(`_TwoStepSDFM_runStaticFM`, T, N, S, Lambda, mu_e, Sigma_e, A, order, quarterfy, corr, beta_param, m, seed, R, burn_in, rescale, parallel)
}

